@@ -30,23 +30,20 @@
 # THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 
-""" Apply JSON-Patches (according to draft 06) """
+""" Apply JSON-Patches (according to draft 08) """
 
 # http://tools.ietf.org/html/draft-ietf-appsawg-json-patch-05
 
 # Will be parsed by setup.py to determine package metadata
 __author__ = 'Stefan Kögl <stefan@skoegl.net>'
-__version__ = '0.10'
+__version__ = '0.11'
 __website__ = 'https://github.com/stefankoegl/python-json-patch'
 __license__ = 'Modified BSD License'
 
 import copy
 import sys
 
-if sys.version_info < (2, 6):
-    import simplejson as json
-else:
-    import json
+import json
 
 import jsonpointer
 
@@ -198,7 +195,7 @@ class JsonPatch(object):
 
 
     def __hash__(self):
-        return hash(frozenset(self.operations))
+        return hash(tuple(self._get_operation(op) for op in self.patch))
 
 
     def __eq__(self, other):
@@ -310,7 +307,6 @@ class JsonPatch(object):
     def _get_operation(self, operation):
         if 'op' not in operation:
             raise JsonPatchException("Operation does not contain 'op' member")
-            raise Exception
 
         op = operation['op']
         if op not in self.operations:
@@ -350,7 +346,10 @@ class RemoveOperation(PatchOperation):
 
     def apply(self, obj):
         subobj, part = self.pointer.to_last(obj)
-        del subobj[part]
+        try:
+            del subobj[part]
+        except KeyError as ex:
+            raise JsonPatchConflict(str(ex))
 
 
 class AddOperation(PatchOperation):
@@ -372,9 +371,6 @@ class AddOperation(PatchOperation):
                 subobj.insert(part, value)
 
         elif isinstance(subobj, dict):
-            if part in subobj:
-                raise JsonPatchConflict("object '%s' already exists" % part)
-
             subobj[part] = value
 
         else:
@@ -409,16 +405,15 @@ class MoveOperation(PatchOperation):
     """Moves an object property or an array element to new location."""
 
     def apply(self, obj):
-        subobj, part = self.pointer.to_last(obj)
+        from_ptr = jsonpointer.JsonPointer(self.operation['from'])
+        subobj, part = from_ptr.to_last(obj)
         value = subobj[part]
 
-        to_ptr = jsonpointer.JsonPointer(self.operation['to'])
-
-        if self.pointer.contains(to_ptr):
+        if from_ptr.contains(self.pointer):
             raise JsonPatchException('Cannot move values into its own children')
 
-        RemoveOperation({'op': 'remove', 'path': self.location}).apply(obj)
-        AddOperation({'op': 'add', 'path': self.operation['to'], 'value': value}).apply(obj)
+        RemoveOperation({'op': 'remove', 'path': self.operation['from']}).apply(obj)
+        AddOperation({'op': 'add', 'path': self.location, 'value': value}).apply(obj)
 
 
 class TestOperation(PatchOperation):
@@ -427,26 +422,25 @@ class TestOperation(PatchOperation):
     def apply(self, obj):
         try:
             subobj, part = self.pointer.to_last(obj)
-            val = self.pointer.walk(subobj, part)
-
-        except JsonPointerException:
-            exc_info = sys.exc_info()
-            exc = JsonPatchTestFailed(str(exc_info[1]))
-            if sys.version_info >= (3, 0):
-                raise exc.with_traceback(exc_info[2])
+            if part is None:
+                val = subobj
             else:
-                raise exc
+                val = self.pointer.walk(subobj, part)
+
+        except JsonPointerException as ex:
+            raise JsonPatchTestFailed(str(ex))
 
         if 'value' in self.operation:
             value = self.operation['value']
             if val != value:
-                raise JsonPatchTestFailed('%s is not equal to tested value %s' % (val, value))
+                raise JsonPatchTestFailed('%s is not equal to tested value %s (types %s and %s)' % (val, value, type(val), type(value)))
 
 
 class CopyOperation(PatchOperation):
     """ Copies an object property or an array element to a new location """
 
     def apply(self, obj):
-        subobj, part = self.pointer.to_last(obj)
+        from_ptr = jsonpointer.JsonPointer(self.operation['from'])
+        subobj, part = from_ptr.to_last(obj)
         value = copy.deepcopy(subobj[part])
-        AddOperation({'op': 'add', 'path': self.operation['to'], 'value': value}).apply(obj)
+        AddOperation({'op': 'add', 'path': self.location, 'value': value}).apply(obj)
